#!/usr/bin/env python
"""
To install this, copy it into this folder:
~/.gimp-2.8/plug-ins

"""
from gimpfu import *
import sys

# A 3x1 box filter. The convmatrix is column major.
boxf = [0]*25
boxf[7:20:5] = [1, 1, 1]

# These matrices are by psycopathicteen and convert RGB to
# NTSC signal levels at 0, 120, and 240 degrees and back
# R sent to R, G sent to G, B sent to R, R sent to G, ...
# https://forums.nesdev.com/viewtopic.php?p=183090#p183090
rgb2ntsc = [.905, .286, -.191, -.160, 1.177, -.017, .152, .298, .550]
ntsc2rgb = [.991, -.325, .334, .130, .800, .070, -.344, -.344, 1.688]
##rgb2ntsc = [.6, .3, .1, .6, .3, .1, .6, .3, .1]
rgb2ntsc = [.8 * c for c in rgb2ntsc]
ntsc2rgb = [1.25 * c for c in ntsc2rgb]

##im = gimp.image_list()[0]
##curlayer = im.active_layer

def snes_ntsc(im, curlayer, fixsar=True, isntsc=True):
    """Apply NTSC filtering"""

    previnterp = pdb.gimp_context_get_interpolation()
    try:
        w, h = im.width, im.height
##        im.disable_undo()
        if im.base_type != RGB:
            pdb.gimp_image_convert_rgb(im)
        pdb.gimp_image_scale_full(im, w * 2, h * 2, INTERPOLATION_NONE)
        if isntsc:
            pdb.plug_in_colors_channel_mixer(im, curlayer, 0, *rgb2ntsc)
        # Video: 3x3 (pattern 5), not additive, rotated
        pdb.plug_in_video(im, curlayer, 5, 0, 1)
        # L, R, G, B, A
        chs = [0, 1, 1, 1, 0]
        pdb.plug_in_convmatrix(im, curlayer, len(boxf), boxf, 0, sum(boxf)/3, 0, len(chs), chs, 2)
        pdb.plug_in_pixelize2(im, curlayer, 1, 2)
        if isntsc:
            pdb.plug_in_colors_channel_mixer(im, curlayer, 0, *ntsc2rgb)
        if fixsar:
            pdb.gimp_image_scale_full(im, int(round(w * 16.0 / 7.0)), h * 2, INTERPOLATION_LINEAR)
    finally:
##        im.enable_undo()
        pdb.gimp_context_set_interpolation(previnterp)

proc_name = 'snes-ntsc'
blurb = "Applies NTSC filtering to an image"
pluginhelp = """
Applies NTSC filtering to an image.

The image is scaled up by 2.28 horizontally and
2 vertically.
"""
author = "Damian Yerrick"
coprnotice = "2016 Damian Yerrick"
date = "2016"
label = "<Image>/Filters/Distorts/NTSC Filter..."
imagetypes = "*"
args = [
##    (PF_IMAGE, "im", "_Image", None),
##    (PF_DRAWABLE, "curlayer", "_Drawable", None),
    (PF_BOOL, "fixsar", "Correct Aspect _Ratio", False),
    (PF_RADIO, "isntsc", "Color space", 0, (
        ("RGB", 0),
        ("NTSC", 1),
    )),
]
results = []

register(
    proc_name, blurb, pluginhelp, author, coprnotice, date,
    label, imagetypes, args, results, snes_ntsc
)
main()
